% function [results_s] = F_multi_label(ground_file, estim_file, affinity_file)
%
% DESCRIPTION:
% ============
% Computation of multi-label classification evaluation scores.
%
% INPUTS:
% =======
% - ground_file:		path to the annotation of the test-set definition text file (ground truth).
% - estim_file:			path to the results on the test-set
% - affinity_file:		path to the affinity on the test-set
%
% OUTPUTS:
% ========
% - result_s			structure containing all evaluation score (recall, precision, f-measure, map, auc, ...)
%
% Damien Tardieu (damien.tardieu@ircam.fr)
% Re-written by Geoffroy Peeters (peeters@ircam.fr) in 2012 for Media-Eval Music Tagging brave new task
%
% Copyright (c) 2012 IRCAM, All Rights Reserved.
% This work was founded by the ?Quaero? Program funded by Oseo French State agency for innovation.

function [results_s] = F_multi_label(ground_file, estim_file, affinity_file)

if ~nargin
	ground_s.file	= './train.csv';
	estim_s.file	= './result_class.csv';
	aff_s.file		= './result_class_affinity.csv';
else
	ground_s.file	= ground_file;
	estim_s.file	= estim_file;
	aff_s.file		= affinity_file;
end

[ground_s.file_c, ground_s.label_c, ground_s.class_m]	= F_read_DefFile(ground_s.file);
[estim_s.file_c, estim_s.label_c, estim_s.class_m]		= F_read_DefFile(estim_s.file);
[aff_s.file_c, aff_s.label_c, aff_s.value_m]			= F_read_AffFile(aff_s.file);
results_s = F_multi_label_compute_scores(ground_s.class_m, estim_s.class_m, aff_s.value_m);
for t=1:length(ground_s.label_c)
	results_s.byTAG(t).name	= ground_s.label_c{t};	
end


% === global results (OVER TAGS)
results_s.Mean = Fgetmean(results_s.byTAG);


% === detailled results by CATEGORIES
map = getMapinv('tags_to_categories.txt');

cat_c = map.keys();
for c=1:length(cat_c)
	tag_c = map(cat_c{c});
	pos_v = [];
	for t=1:length(tag_c)
		pos_v(t) = find(ismember({results_s.byTAG.name}, tag_c{t}));
	end
	results_s.byCAT(c)		= Fgetmean(results_s.byTAG(pos_v));
end
for c=1:length(cat_c)
	results_s.byCAT(c).name	= cat_c{c};	
end




% ==================================
function [Mean] = Fgetmean(byTAG)

fieldname_c = fieldnames(byTAG);
for f=1:length(fieldname_c)
	Mean.(fieldname_c{f}) = mean([byTAG.(fieldname_c{f})]);
end




% function [file_c, label_c, class_m] = F_read_DefFile(def_file, [ref_file_c], [ref_label_c])
%
% DESCRIPTION:
% ============
% read file/class Definition file
%
% INPUTS:
% =======
% - def_file:		full-path to text-file in the following format
%		"filename \t label \n filename \t label \n ..."
% - optional [ref_file_c]: to force the mapping to a predifined set of file
% - optional [ref_label_c]: to force the mapping to a predifined set of labels
%
% OUTPUTS:
% ========
% file_c	(nb_file)
% label_c	(nb_label)
% class_m	(nb_file, nb_label)
%
% Damien Tardieu (damien.tardieu@ircam.fr)
% Re-written by Geoffroy Peeters (peeters@ircam.fr) in 2012 for Media-Eval Music Tagging brave new task
%
% Copyright (c) 2012 IRCAM, All Rights Reserved.
% This work was founded by the ?Quaero? Program funded by Oseo French State agency for innovation.

function [file_c, label_c, class_m] = F_read_DefFile(def_file, ref_file_c, ref_label_c)

data_c			= textread(def_file, '%s', 'endofline', '\n', 'delimiter', '\t');
file_c			= data_c(1:2:end);
label_c			= data_c(2:2:end);
if nargin>1 & ~isempty(ref_file_c),	list_file_c		= ref_file_c;
else								list_file_c		= unique(file_c);
end
if nargin>2 & ~isempty(ref_label_c),list_label_c	= ref_label_c;
else								list_label_c	= unique(label_c);
end

map = containers.Map(list_label_c, [1:length(list_label_c)]);

class_m = zeros(length(list_file_c), length(list_label_c));

for num_file=1:length(list_file_c)
	pos_file_v = find(ismember(file_c, list_file_c{num_file}));
	if isempty(pos_file_v), fprintf(1, 'file %s does not exist in def_file\n', list_file_c{num_file}); end
	for p=1:length(pos_file_v)
		class_m(num_file, map(label_c{pos_file_v(p)})) = class_m(num_file, map(label_c{pos_file_v(p)}))+1;
	end
end

file_c	= list_file_c;
label_c = list_label_c;




% function [file_c, label_c, affinity_m] = F_read_AffFile(def_file, [ref_file_c], [ref_label_c])
%
% DESCRIPTION:
% ============
% read file/class/affinity Affinity file
%
% INPUTS:
% =======
% - def_file:		full-path to text-file in the following format
%		"filename \t label \n filename \t label \n ..."
% - optional [ref_file_c]: to force the mapping to a predifined set of file
% - optional [ref_label_c]: to force the mapping to a predifined set of labels
%
% OUTPUTS:
% ========
% file_c		(nb_file)
% label_c		(nb_label)
% affinity_m	(nb_file, nb_label)
%
% Damien Tardieu (damien.tardieu@ircam.fr)
% Re-written by Geoffroy Peeters (peeters@ircam.fr) in 2012 for Media-Eval Music Tagging brave new task
%
% Copyright (c) 2012 IRCAM, All Rights Reserved.
% This work was founded by the ?Quaero? Program funded by Oseo French State agency for innovation.

function [file_c, label_c, affinity_m] = F_read_AffFile(def_file, ref_file_c, ref_label_c)

data_c		= textread(def_file, '%s', 'endofline', '\n', 'delimiter', '\t');
file_c		= data_c(1:3:end);
label_c		= data_c(2:3:end);
affinity_c	= data_c(3:3:end);

if nargin>1 & ~isempty(ref_file_c),	list_file_c		= ref_file_c;
else								list_file_c		= unique(file_c);
end
if nargin>2 & ~isempty(ref_label_c),list_label_c	= ref_label_c;
else								list_label_c	= unique(label_c);
end

map = containers.Map(list_label_c, [1:length(list_label_c)]);

affinity_m = zeros(length(list_file_c), length(list_label_c));

for num_file=1:length(list_file_c)
	pos_file_v = find(ismember(file_c, list_file_c{num_file}));
	if isempty(pos_file_v), fprintf(1, 'file %s does not exist in def_file\n', list_file_c{num_file}); end
	for p=1:length(pos_file_v)
		affinity_m(num_file, map(label_c{pos_file_v(p)})) = str2double(affinity_c{pos_file_v(p)});
	end
end

file_c	= list_file_c;
label_c = list_label_c;
